document.addEventListener('DOMContentLoaded', function() {
    // Tab switching functionality
    const generatorTab = document.getElementById('linkmap-generator-tab');
    const visualizationTab = document.getElementById('linkmap-visualization-tab');
    const generatorContent = document.getElementById('linkmap-generator');
    const visualizationContent = document.getElementById('linkmap-visualization');
    
    if (generatorTab && visualizationTab) {
        generatorTab.addEventListener('click', function(e) {
            e.preventDefault();
            generatorTab.classList.add('nav-tab-active');
            visualizationTab.classList.remove('nav-tab-active');
            generatorContent.style.display = 'block';
            visualizationContent.style.display = 'none';
        });
        
        visualizationTab.addEventListener('click', function(e) {
            e.preventDefault();
            visualizationTab.classList.add('nav-tab-active');
            generatorTab.classList.remove('nav-tab-active');
            visualizationContent.style.display = 'block';
            generatorContent.style.display = 'none';
            
            // Load visualization data if not already loaded
            if (!window.linkLoomGraphLoaded) {
                loadVisualizationData();
            }
        });
    }

    // Link Generator functionality
    const form = document.getElementById('linkmap-form');
    if (form) {
        const radios = form.getElementsByClassName('mapping-radio');
        const groups = form.getElementsByClassName('linkmap-group');
        
        function updateOptions() {
            // Get all selected target URLs and keywords
            const selectedUrls = new Set();
            const selectedKeywords = new Set();
            Array.from(radios).forEach(radio => {
                if (radio.checked) {
                    const option = radio.closest('.mapping-option');
                    selectedUrls.add(option.dataset.toUrl);
                    selectedKeywords.add(option.dataset.keyword);
                }
            });
            
            // Update options based on selections
            Array.from(document.getElementsByClassName('mapping-option')).forEach(option => {
                if (option.classList.contains('is-already-mapped')) {
                    return; // Skip already mapped options
                }

                const toUrl = option.dataset.toUrl;
                const keyword = option.dataset.keyword;
                const radio = option.querySelector('.mapping-radio');
                
                if (selectedUrls.has(toUrl) && !radio.checked) {
                    option.classList.add('is-disabled');
                    option.setAttribute('data-disabled-reason', linkloomData.i18n.urlAlreadySelected);
                } else if (selectedKeywords.has(keyword) && !radio.checked) {
                    option.classList.add('is-disabled');
                    option.setAttribute('data-disabled-reason', linkloomData.i18n.keywordAlreadySelected);
                } else {
                    option.classList.remove('is-disabled');
                    option.removeAttribute('data-disabled-reason');
                }
            });

            // Update clear selection buttons
            Array.from(groups).forEach(group => {
                const groupId = group.dataset.groupId;
                const hasSelection = Array.from(group.querySelectorAll('.mapping-radio')).some(radio => radio.checked);
                const clearButton = group.querySelector('.clear-selection');
                if (hasSelection) {
                    clearButton.classList.remove('linkloom-hidden');
                } else {
                    clearButton.classList.add('linkloom-hidden');
                }
            });
        }
        
        // Add change event listeners to all radio buttons
        Array.from(radios).forEach(radio => {
            radio.addEventListener('change', updateOptions);
        });

        // Add click handlers for clear selection buttons
        Array.from(document.getElementsByClassName('clear-selection')).forEach(button => {
            button.addEventListener('click', function(e) {
                e.preventDefault();
                const group = this.closest('.linkmap-group');
                const radios = group.querySelectorAll('.mapping-radio');
                radios.forEach(radio => {
                    radio.checked = false;
                });
                updateOptions();
            });
        });
        
        // Initial update
        updateOptions();
    }

    // Visualization functionality
    function loadVisualizationData() {
        const graphContainer = document.getElementById('linkmap-graph');
        if (!graphContainer) return;
        
        // Show loading indicator
        graphContainer.innerHTML = '<div class="spinner is-active" style="float:none;margin:0 auto;display:block;margin-top:200px;"></div>';
        
        // Fetch data via AJAX
        fetch(linkloomData.ajaxUrl, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded',
            },
            body: new URLSearchParams({
                action: 'linkloom_get_visualization_data',
                nonce: linkloomData.nonce
            })
        })
        .then(response => response.json())
        .then(response => {
            if (response.success && response.data) {
                renderVisualization(response.data);
                window.linkLoomGraphLoaded = true;
            } else {
                graphContainer.innerHTML = '<p class="error">' + linkloomData.i18n.visualizationError + '</p>';
            }
        })
        .catch(error => {
            console.error('Error loading visualization data:', error);
            graphContainer.innerHTML = '<p class="error">' + linkloomData.i18n.visualizationError + '</p>';
        });
    }

    function renderVisualization(data) {
        const graphContainer = document.getElementById('linkmap-graph');
        if (!graphContainer) return;
        
        // Format nodes and edges for vis.js
        const nodes = new vis.DataSet(data.nodes.map(node => ({
            id: node.id,
            label: node.label,
            title: node.url,
            color: node.type === 'post' ? '#2271b1' : '#3582c4',
            shape: 'dot',
            size: 10
        })));
        
        const edges = new vis.DataSet(data.edges.map(edge => ({
            from: edge.from,
            to: edge.to,
            arrows: 'to',
            color: {color: '#72777c', opacity: 0.8},
            width: 1
        })));
        
        // Create network
        const networkData = {
            nodes: nodes,
            edges: edges
        };
        
        const options = {
            physics: {
                stabilization: true,
                barnesHut: {
                    gravitationalConstant: -2000,
                    centralGravity: 0.1,
                    springLength: 150,
                    springConstant: 0.05
                }
            },
            layout: {
                improvedLayout: true
            },
            interaction: {
                navigationButtons: true,
                keyboard: true
            }
        };
        
        const network = new vis.Network(graphContainer, networkData, options);
        
        // Add zoom controls
        document.getElementById('linkmap-zoom-in').addEventListener('click', function() {
            network.zoomIn(0.2);
        });
        
        document.getElementById('linkmap-zoom-out').addEventListener('click', function() {
            network.zoomOut(0.2);
        });
        
        document.getElementById('linkmap-reset').addEventListener('click', function() {
            network.fit();
        });
        
        // Store network in window for debugging
        window.linkLoomNetwork = network;
    }
});
