<?php

/**
 * LinkMap page rendering for Link Loom
 *
 * @package LinkLoom
 * @subpackage Admin
 */

if (!defined('ABSPATH')) {
    exit;
}

/**
 * Render the LinkMap page.
 */
function linkloom_render_linkmap_page() {
    if (!current_user_can('manage_options')) {
        wp_die(esc_html__('Unauthorized user', 'link-loom-seo-internal-link-builder'));
    }

    echo '<div class="wrap linkloom-admin-page">';
    echo '<h1>' . esc_html__('LinkMap', 'link-loom-seo-internal-link-builder') . '</h1>';
    
    // Add tab navigation
    echo '<div class="nav-tab-wrapper">';
    echo '<a href="#linkmap-generator" class="nav-tab nav-tab-active" id="linkmap-generator-tab">' . 
        esc_html__('Link Generator', 'link-loom-seo-internal-link-builder') . '</a>';
    echo '<a href="#linkmap-visualization" class="nav-tab" id="linkmap-visualization-tab">' . 
        esc_html__('Link Structure', 'link-loom-seo-internal-link-builder') . '</a>';
    echo '</div>';
    
    // Add tab content containers
    echo '<div id="linkmap-generator" class="linkmap-tab-content">';

    if (isset($_POST['linkmap_generate'])) {
        check_admin_referer('linkloom_linkmap');
        $candidates = linkloom_generate_linkmap();
        
        if (!empty($candidates)) {
            // Get existing mappings
            $settings = get_option('linkloom_settings', array());
            $existing_mappings = isset($settings['mappings']) ? $settings['mappings'] : array();
            $existing_keywords = array_keys($existing_mappings);

            // Group candidates by source page
            $grouped_candidates = array();
            foreach ($candidates as $cand) {
                $page_id = $cand['page_id'];
                if (!isset($grouped_candidates[$page_id])) {
                    $grouped_candidates[$page_id] = array(
                        'page_title' => $cand['page_title'],
                        'candidates' => array()
                    );
                }
                $grouped_candidates[$page_id]['candidates'][] = $cand;
            }

            // Styles are now enqueued via wp_enqueue_style in menu.php
            ?>

            <form method="post" id="linkmap-form">
            <?php wp_nonce_field('linkloom_linkmap'); ?>
            
            <?php
            // Output each group
            foreach ($grouped_candidates as $page_id => $group) {
                echo '<div class="linkmap-group" data-group-id="' . esc_attr($page_id) . '">';
                echo '<div class="linkmap-group-header">';
                echo '<h3 class="linkmap-group-title">' . esc_html($group['page_title']) . '</h3>';
                echo '<a href="#" class="clear-selection linkloom-hidden"><span class="dashicons dashicons-dismiss"></span> ' . esc_html__('Clear Selection', 'link-loom-seo-internal-link-builder') . '</a>';
                echo '</div>';
                echo '<div class="linkmap-candidates">';
                
                foreach ($group['candidates'] as $cand) {
                    $option_id = 'option_' . intval($page_id) . '_' . md5($cand['urlto']);
                    $is_already_mapped = in_array($cand['keyword'], $existing_keywords);
                    $option_classes = array('mapping-option');
                    if ($is_already_mapped) {
                        $option_classes[] = 'is-already-mapped';
                    }
                    
                    echo '<div class="' . esc_attr(implode(' ', $option_classes)) . '" ' .
                         'data-to-url="' . esc_attr($cand['urlto']) . '" ' .
                         'data-keyword="' . esc_attr($cand['keyword']) . '">';
                    
                    // Add status message for already mapped keywords
                    if ($is_already_mapped) {
                        echo '<span class="mapping-status-message">' . esc_html__('This keyword is already mapped elsewhere', 'link-loom-seo-internal-link-builder') . '</span>';
                    }
                    
                    // Keyword header
                    echo '<div class="mapping-header">';
                    echo '<h4 class="mapping-keyword">' . esc_html($cand['keyword']) . '</h4>';
                    echo '</div>';
                    
                    // Content section
                    echo '<div class="mapping-content">';
                    echo '<div class="mapping-url">';
                    echo '<span class="dashicons dashicons-admin-links"></span> ';
                    echo '<a href="' . esc_url($cand['urlto']) . '" target="_blank" rel="noopener noreferrer">' . esc_html($cand['urlto']) . '</a>';
                    echo '</div>';
                    
                    // Selection radio
                    echo '<div class="mapping-select">';
                    echo '<input type="radio" id="' . esc_attr($option_id) . '" name="selected_mapping[' . intval($page_id) . ']" value="' . 
                        esc_attr($cand['keyword'] . '|' . $cand['urlto'] . '|' . $cand['urlfrom']) . 
                        '" class="mapping-radio" data-option-id="' . esc_attr($option_id) . '"' .
                        ($is_already_mapped ? ' disabled' : '') . '>';
                    echo '<label for="' . esc_attr($option_id) . '">' . esc_html__('Select this mapping', 'link-loom-seo-internal-link-builder') . '</label>';
                    echo '</div>';
                    echo '</div>';
                    
                    echo '</div>';
                }
                
                echo '</div>';
                echo '</div>';
            }
            ?>
            
            <p><input type="submit" name="linkmap_submit" class="button-primary" value="<?php echo esc_attr__('Save Mappings', 'link-loom-seo-internal-link-builder'); ?>"></p>
            </form>
            
            <?php
            // JavaScript is now enqueued via wp_enqueue_script in menu.php
        } else {
            echo '<div class="notice notice-info"><p>' . esc_html__('No candidate mappings found.', 'link-loom-seo-internal-link-builder') . '</p></div>';
        }
    } elseif (isset($_POST['linkmap_submit'])) {
        check_admin_referer('linkloom_linkmap');
        
        $selected = isset($_POST['selected_mapping']) ? 
            map_deep(wp_unslash($_POST['selected_mapping']), 'sanitize_text_field') : array();
        
        $settings = get_option('linkloom_settings', array());
        if (!isset($settings['mappings'])) {
            $settings['mappings'] = array();
        }
        
        foreach ($selected as $mapping_str) {
            $parts = explode('|', $mapping_str);
            if (count($parts) === 3) {
                $keyword = sanitize_text_field($parts[0]);
                $urlto = esc_url_raw(trim($parts[1]));
                $urlfrom = esc_url_raw(trim($parts[2]));
                $settings['mappings'][$keyword] = array('urlto' => $urlto, 'urlfrom' => $urlfrom);
            }
        }
        
        update_option('linkloom_settings', $settings);
        echo '<div class="notice notice-success"><p>' . esc_html__('LinkMap mappings saved.', 'link-loom-seo-internal-link-builder') . '</p></div>';
        echo '<p><a href="' . esc_url(admin_url('admin.php?page=linkloom-settings')) . '" class="button">' . esc_html__('Return to Settings', 'link-loom-seo-internal-link-builder') . '</a></p>';
    } else {
        echo '<form method="post">';
        wp_nonce_field('linkloom_linkmap');
        echo '<p><input type="submit" name="linkmap_generate" class="button-primary" value="' . esc_attr__('Generate Mappings', 'link-loom-seo-internal-link-builder') . '"></p>';
        echo '</form>';
    }
    
    echo '</div>'; // Close linkmap-generator div
    
    // Add visualization tab content
    echo '<div id="linkmap-visualization" class="linkmap-tab-content" style="display:none;">';
    echo '<div class="linkmap-visualization-container">';
    echo '<div id="linkmap-graph"></div>';
    echo '<div class="linkmap-visualization-legend">';
    echo '<h3>' . esc_html__('Legend', 'link-loom-seo-internal-link-builder') . '</h3>';
    echo '<div class="legend-item"><span class="legend-color post-color"></span>' . 
        esc_html__('Posts', 'link-loom-seo-internal-link-builder') . '</div>';
    echo '<div class="legend-item"><span class="legend-color page-color"></span>' . 
        esc_html__('Pages', 'link-loom-seo-internal-link-builder') . '</div>';
    echo '<div class="legend-item"><span class="legend-arrow"></span>' . 
        esc_html__('Link Direction', 'link-loom-seo-internal-link-builder') . '</div>';
    echo '</div>';
    echo '<div class="linkmap-visualization-controls">';
    echo '<button type="button" class="button" id="linkmap-zoom-in"><span class="dashicons dashicons-plus"></span></button>';
    echo '<button type="button" class="button" id="linkmap-zoom-out"><span class="dashicons dashicons-minus"></span></button>';
    echo '<button type="button" class="button" id="linkmap-reset"><span class="dashicons dashicons-image-rotate"></span></button>';
    echo '</div>';
    echo '</div>';
    echo '</div>'; // Close linkmap-visualization div
    
    echo '</div>'; // Close wrap div
}
