<?php

/**
 * Admin menu registration for Link Loom
 *
 * @package LinkLoom
 * @subpackage Admin
 */

if (!defined('ABSPATH')) {
    exit;
}

/**
 * Register the admin menu and subpages.
 *
 * The top-level menu is "Link Loom" and its submenus are "Settings", "Documentation", and "LinkMap".
 */
function linkloom_register_menu_global() {
    // Add top-level menu
    add_menu_page(
        'Link Loom: SEO Internal Link Builder',  // Page title
        'Link Loom',                             // Menu title
        'manage_options',                        // Capability
        'linkloom-settings',                     // Menu slug
        'linkloom_render_admin_page_global',     // Callback function (Settings page)
        'dashicons-admin-links',                 // Icon
        65                                       // Position
    );

    // Add "Settings" submenu (overrides the top-level page label)
    add_submenu_page(
        'linkloom-settings',                     // Parent slug
        'Settings',                              // Page title
        'Settings',                              // Menu title
        'manage_options',                        // Capability
        'linkloom-settings',                     // Menu slug (same as parent)
        'linkloom_render_admin_page_global'      // Callback function
    );

    // Add "Documentation" submenu
    add_submenu_page(
        'linkloom-settings',                     // Parent slug
        'Documentation',                         // Page title
        'Documentation',                         // Menu title
        'manage_options',                        // Capability
        'linkloom-documentation',                // Menu slug
        'linkloom_render_documentation_page'     // Callback function
    );

    // Add "LinkMap" submenu
    add_submenu_page(
        'linkloom-settings',                     // Parent slug
        'LinkMap',                              // Page title
        'LinkMap',                              // Menu title
        'manage_options',                        // Capability
        'linkloom-linkmap',                     // Menu slug
        'linkloom_render_linkmap_page'          // Callback function
    );
}
add_action('admin_menu', 'linkloom_register_menu_global');

/**
 * Enqueue admin styles and scripts.
 */
function linkloom_admin_enqueue_assets() {
    // Only load on our plugin pages
    $screen = get_current_screen();
    if ($screen !== null && $screen->base !== null && strpos($screen->base, 'linkloom') !== false) {
        // Enqueue WordPress styles
        wp_enqueue_style('dashicons');
        
        // Register and enqueue common CSS for all plugin pages
        wp_register_style(
            'linkloom-common-style',
            LINKLOOM_PLUGIN_URL . 'assets/css/linkloom-common.css',
            array(),
            LINKLOOM_VERSION
        );
        wp_enqueue_style('linkloom-common-style');
        
        
        // Register page-specific styles
        wp_register_style(
            'linkloom-linkmap-style',
            LINKLOOM_PLUGIN_URL . 'assets/css/linkmap.css',
            array('linkloom-common-style'),
            LINKLOOM_VERSION
        );
        
        // Register scripts
        wp_register_script(
            'linkloom-settings-script',
            LINKLOOM_PLUGIN_URL . 'assets/js/settings.js',
            array('jquery'),
            LINKLOOM_VERSION,
            true
        );
        
        wp_register_script(
            'linkloom-linkmap-script',
            LINKLOOM_PLUGIN_URL . 'assets/js/linkmap.js',
            array('jquery'),
            LINKLOOM_VERSION,
            true
        );
        
        // Conditionally enqueue based on the current page
        if ($screen->base === 'link-loom_page_linkloom-linkmap') {
            wp_enqueue_style('linkloom-linkmap-style');
            wp_enqueue_script('linkloom-linkmap-script');
            
            // Pass data to JavaScript
            // Register vis.js from CDN
            wp_register_script(
                'vis-js',
                'https://cdnjs.cloudflare.com/ajax/libs/vis/4.21.0/vis.min.js',
                array(),
                '4.21.0',
                true
            );
            wp_register_style(
                'vis-css',
                'https://cdnjs.cloudflare.com/ajax/libs/vis/4.21.0/vis.min.css',
                array(),
                '4.21.0'
            );

            // Enqueue vis.js for LinkMap page
            wp_enqueue_script('vis-js');
            wp_enqueue_style('vis-css');

            wp_localize_script('linkloom-linkmap-script', 'linkloomData', array(
                'ajaxUrl' => admin_url('admin-ajax.php'),
                'nonce' => wp_create_nonce('linkloom-linkmap-nonce'),
                'i18n' => array(
                    'urlAlreadySelected' => esc_html__('This target URL is already selected', 'link-loom-seo-internal-link-builder'),
                    'keywordAlreadySelected' => esc_html__('This keyword is already selected', 'link-loom-seo-internal-link-builder'),
                    'keywordAlreadyMapped' => esc_html__('This keyword is already mapped elsewhere', 'link-loom-seo-internal-link-builder'),
                    'visualizationError' => esc_html__('Error loading visualization data', 'link-loom-seo-internal-link-builder'),
                    'loadingVisualization' => esc_html__('Loading link structure visualization...', 'link-loom-seo-internal-link-builder')
                )
            ));
        } elseif ($screen->base === 'toplevel_page_linkloom-settings') {
            wp_enqueue_script('linkloom-settings-script');
            
            // Example of using wp_localize_script to pass data to JavaScript
            wp_localize_script('linkloom-settings-script', 'linkloomData', array(
                'ajaxUrl' => admin_url('admin-ajax.php'),
                'nonce' => wp_create_nonce('linkloom-settings-nonce')
            ));
            
            // Example of using wp_add_inline_script for small script snippets
            wp_add_inline_script('linkloom-settings-script', 'console.log("Link Loom Settings page loaded");', 'before');
        }
    }
}
add_action('admin_enqueue_scripts', 'linkloom_admin_enqueue_assets');
