<?php

/**
 * Settings page rendering for Link Loom
 *
 * @package LinkLoom
 * @subpackage Admin
 */

if (!defined('ABSPATH')) {
    exit;
}

/**
 * Render the Settings page.
 */
function linkloom_render_admin_page_global() {
    if (!current_user_can('manage_options')) {
        wp_die(esc_html__('Unauthorized user', 'link-loom-seo-internal-link-builder'));
    }

    if (isset($_POST['linkloom_submit'])) {
        check_admin_referer('linkloom_save_settings');
        
        // Sanitize the raw mappings input
        $mappings_raw = isset($_POST['linkloom_mappings_raw']) ? 
            sanitize_textarea_field(wp_unslash($_POST['linkloom_mappings_raw'])) : '';
        
        // Process mappings using sanitization utility
        $mappings_array = linkloom_sanitize_mappings($mappings_raw);
        
        // Gather and sanitize other settings
        $input = array(
            'mappings' => $mappings_array,
            'instance_selection' => isset($_POST['linkloom_instance_selection']) ? 
                sanitize_text_field(wp_unslash($_POST['linkloom_instance_selection'])) : 'first',
            'global_limit_mode' => isset($_POST['linkloom_global_limit_mode']) ? 
                sanitize_text_field(wp_unslash($_POST['linkloom_global_limit_mode'])) : 'fixed',
            'global_limit_fixed' => isset($_POST['linkloom_global_limit_fixed']) ? 
                intval($_POST['linkloom_global_limit_fixed']) : 1,
            'global_limit_min' => isset($_POST['linkloom_global_limit_min']) ? 
                intval($_POST['linkloom_global_limit_min']) : 1,
            'global_limit_max' => isset($_POST['linkloom_global_limit_max']) ? 
                intval($_POST['linkloom_global_limit_max']) : 1,
        );
        
        // Validate and sanitize all settings
        $settings = linkloom_sanitize_settings($input);
        
        update_option('linkloom_settings', $settings);
        echo '<div class="updated"><p>' . esc_html__('Settings saved.', 'link-loom-seo-internal-link-builder') . '</p></div>';
    }

    // Get current settings with defaults
    $settings = get_option('linkloom_settings', array(
        'mappings' => array(),
        'instance_selection' => 'first',
        'global_limit_mode' => 'fixed',
        'global_limit_fixed' => 1,
        'global_limit_min' => 1,
        'global_limit_max' => 1,
    ));

    // Convert mappings to raw format for textarea
    $mappings_raw = '';
    if (!empty($settings['mappings'])) {
        foreach ($settings['mappings'] as $keyword => $value) {
            if (is_array($value)) {
                $mappings_raw .= $keyword . '|' . $value['urlto'] . '|' . $value['urlfrom'] . "\n";
            } else {
                $mappings_raw .= $keyword . '|' . $value . "\n";
            }
        }
    }

    // Get other settings
    $instance_selection = isset($settings['instance_selection']) ? $settings['instance_selection'] : 'first';
    $global_limit_mode = isset($settings['global_limit_mode']) ? $settings['global_limit_mode'] : 'fixed';
    $global_limit_fixed = isset($settings['global_limit_fixed']) ? intval($settings['global_limit_fixed']) : 1;
    $global_limit_min = isset($settings['global_limit_min']) ? intval($settings['global_limit_min']) : 1;
    $global_limit_max = isset($settings['global_limit_max']) ? intval($settings['global_limit_max']) : 1;
    ?>
    <div class="wrap linkloom-admin-page">
        <h1><?php echo esc_html__('Link Loom: SEO Internal Link Builder Settings', 'link-loom-seo-internal-link-builder'); ?></h1>
        <form method="post">
            <?php wp_nonce_field('linkloom_save_settings'); ?>
            <table class="form-table">
                <tr valign="top">
                    <th scope="row"><label for="linkloom_mappings_raw"><?php echo esc_html__('Keyword Mappings', 'link-loom-seo-internal-link-builder'); ?></label></th>
                    <td>
                        <textarea id="linkloom_mappings_raw" name="linkloom_mappings_raw" rows="10" cols="50" class="large-text code" aria-describedby="linkloom_mappings_raw_desc"><?php echo esc_textarea($mappings_raw); ?></textarea>
                        <p class="description" id="linkloom_mappings_raw_desc">
                            <?php echo esc_html__('Enter one mapping per line in the format: "keyword|URL" or "keyword|URLto|URLfrom" (do not include the quotes)', 'link-loom-seo-internal-link-builder'); ?>
                        </p>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row"><label for="linkloom_instance_selection"><?php echo esc_html__('Instance Selection', 'link-loom-seo-internal-link-builder'); ?></label></th>
                    <td>
                        <select id="linkloom_instance_selection" name="linkloom_instance_selection" aria-describedby="linkloom_instance_selection_desc">
                            <option value="first" <?php selected($instance_selection, 'first'); ?>><?php echo esc_html__('First Instance', 'link-loom-seo-internal-link-builder'); ?></option>
                            <option value="last" <?php selected($instance_selection, 'last'); ?>><?php echo esc_html__('Last Instance', 'link-loom-seo-internal-link-builder'); ?></option>
                            <option value="random" <?php selected($instance_selection, 'random'); ?>><?php echo esc_html__('Random Instance', 'link-loom-seo-internal-link-builder'); ?></option>
                        </select>
                        <p class="description" id="linkloom_instance_selection_desc"><?php echo esc_html__('Choose which occurrence of each keyword should be linked.', 'link-loom-seo-internal-link-builder'); ?></p>
                    </td>
                </tr>
                <tr valign="top">
                    <th scope="row"><label for="linkloom_global_limit_mode"><?php echo esc_html__('Global Limit Mode', 'link-loom-seo-internal-link-builder'); ?></label></th>
                    <td>
                        <select id="linkloom_global_limit_mode" name="linkloom_global_limit_mode" aria-describedby="linkloom_global_limit_mode_desc">
                            <option value="fixed" <?php selected($global_limit_mode, 'fixed'); ?>><?php echo esc_html__('Fixed', 'link-loom-seo-internal-link-builder'); ?></option>
                            <option value="random" <?php selected($global_limit_mode, 'random'); ?>><?php echo esc_html__('Random', 'link-loom-seo-internal-link-builder'); ?></option>
                        </select>
                        <p class="description" id="linkloom_global_limit_mode_desc"><?php echo esc_html__('Choose whether the total number of distinct keyword links is fixed or random.', 'link-loom-seo-internal-link-builder'); ?></p>
                    </td>
                </tr>
                <tr valign="top" id="linkloom_global_limit_fixed_row">
                    <th scope="row"><label for="linkloom_global_limit_fixed"><?php echo esc_html__('Global Link Count (Fixed)', 'link-loom-seo-internal-link-builder'); ?></label></th>
                    <td>
                        <input type="number" id="linkloom_global_limit_fixed" name="linkloom_global_limit_fixed" 
                               value="<?php echo esc_attr($global_limit_fixed); ?>" min="0" aria-describedby="linkloom_global_limit_fixed_desc">
                        <p class="description" id="linkloom_global_limit_fixed_desc"><?php echo esc_html__('Enter the fixed maximum number of distinct keyword links to insert (0 for no limit).', 'link-loom-seo-internal-link-builder'); ?></p>
                    </td>
                </tr>
                <tr valign="top" id="linkloom_global_limit_random_row" class="linkloom-hidden">
                    <th scope="row"><?php echo esc_html__('Global Link Count (Random)', 'link-loom-seo-internal-link-builder'); ?></th>
                    <td>
                        <input type="number" id="linkloom_global_limit_min" name="linkloom_global_limit_min" 
                               value="<?php echo esc_attr($global_limit_min); ?>" min="0" aria-describedby="linkloom_global_limit_random_desc">
                        <span aria-hidden="true"> to </span>
                        <span class="screen-reader-text"><?php echo esc_html__('to', 'link-loom-seo-internal-link-builder'); ?></span>
                        <input type="number" id="linkloom_global_limit_max" name="linkloom_global_limit_max" 
                               value="<?php echo esc_attr($global_limit_max); ?>" min="0" aria-describedby="linkloom_global_limit_random_desc">
                        <p class="description" id="linkloom_global_limit_random_desc"><?php echo esc_html__('The global link count will be a deterministic random number between these values.', 'link-loom-seo-internal-link-builder'); ?></p>
                    </td>
                </tr>
            </table>
            <?php submit_button(esc_html__('Save Settings', 'link-loom-seo-internal-link-builder'), 'primary', 'linkloom_submit'); ?>
        </form>
    </div>
    <?php
}
