<?php

/**
 * Content processing functions for Link Loom
 *
 * @package LinkLoom
 * @subpackage Core
 */

if (!defined('ABSPATH')) {
    exit;
}

/**
 * Filter post content to link a single occurrence per keyword and limit total distinct keyword links.
 *
 * @param string $content The post content to process.
 * @return string The processed content with added links.
 */
function linkloom_filter_content_global($content) {
    global $post;
    
    // Retrieve settings (with defaults)
    $settings = get_option('linkloom_settings', array(
        'mappings' => array(),
        'instance_selection' => 'first',
        'global_limit_mode' => 'fixed',
        'global_limit_fixed' => 1,
        'global_limit_min' => 1,
        'global_limit_max' => 1,
    ));
    
    $mappings = !empty($settings['mappings']) ? $settings['mappings'] : array();
    if (empty($mappings)) {
        return $content;
    }

    // Check cache first
    $cache_key = linkloom_get_cache_key($post->ID, $settings);
    $cached_content = get_transient($cache_key);
    if ($cached_content !== false) {
        return $cached_content;
    }

    // Split the content into text and HTML parts
    $parts = preg_split('/(<[^>]+>)/', $content, -1, PREG_SPLIT_DELIM_CAPTURE);
    $candidates = array();

    foreach ($mappings as $keyword => $map) {
        if (trim($keyword) === '' || empty($map)) {
            continue;
        }

        // Determine URL based on mapping type
        if (is_array($map)) {
            $url = $map['urlto'];
            // Only add mapping if current page permalink matches the 'from' URL
            if (get_permalink($post->ID) !== $map['urlfrom']) {
                continue;
            }
        } else {
            $url = $map;
        }

        $occurrences = array();
        $current_global_offset = 0;

        foreach ($parts as $index => $part) {
            if (preg_match('/^<[^>]+>$/', $part)) {
                continue;
            }

            // Regex pattern that handles both single words and phrases
            // For phrases, ensure word boundaries and optional spaces between words
            $pattern = '/\\b';
            if ($keyword !== null && strpos($keyword, ' ') !== false) {
                // For phrases, split words and create pattern with flexible spacing
                $phrase_words = explode(' ', preg_quote($keyword, '/'));
                $pattern .= '(' . implode('\\s+', $phrase_words) . ')';
            } else {
                // For single words, use the original pattern
                $pattern .= '(' . preg_quote($keyword, '/') . ')';
            }
            $pattern .= '(?=$|[^\\w\'])/i';

            if (preg_match_all($pattern, $part, $matches, PREG_OFFSET_CAPTURE)) {
                foreach ($matches[0] as $match) {
                    $occurrences[] = array(
                        'part_index' => $index,
                        'local_offset' => $match[1],
                        'match_text' => $match[0],
                        'global_offset' => $current_global_offset + $match[1],
                    );
                }
            }
            $current_global_offset += strlen($part);
        }

        if (empty($occurrences)) {
            continue;
        }

        // Select occurrence based on instance selection mode
        $mode = isset($settings['instance_selection']) ? $settings['instance_selection'] : 'first';
        if ($mode === 'first') {
            usort($occurrences, function($a, $b) {
                return $a['global_offset'] - $b['global_offset'];
            });
            $candidate = $occurrences[0];
        } elseif ($mode === 'last') {
            usort($occurrences, function($a, $b) {
                return $b['global_offset'] - $a['global_offset'];
            });
            $candidate = $occurrences[0];
        } elseif ($mode === 'random') {
            $num_occ = count($occurrences);
            $seed = isset($post->ID) ? $post->ID : 0;
            $hash = md5($seed . $keyword);
            $hash_num = hexdec(substr($hash, 0, 8));
            $rand_index = $hash_num % $num_occ;
            $candidate = $occurrences[$rand_index];
        } else {
            usort($occurrences, function($a, $b) {
                return $a['global_offset'] - $b['global_offset'];
            });
            $candidate = $occurrences[0];
        }

        $candidates[] = array_merge($candidate, array(
            'keyword' => $keyword,
            'url' => $url,
        ));
    }

    // Sort candidates by global offset
    usort($candidates, function($a, $b) {
        return $a['global_offset'] - $b['global_offset'];
    });

    // Apply global limit
    $global_limit_mode = isset($settings['global_limit_mode']) ? $settings['global_limit_mode'] : 'fixed';
    if ($global_limit_mode === 'fixed') {
        $global_limit = isset($settings['global_limit_fixed']) ? intval($settings['global_limit_fixed']) : 1;
    } elseif ($global_limit_mode === 'random') {
        $min = isset($settings['global_limit_min']) ? intval($settings['global_limit_min']) : 1;
        $max = isset($settings['global_limit_max']) ? intval($settings['global_limit_max']) : 1;
        if ($max < $min) {
            $max = $min;
        }
        $seed = isset($post->ID) ? $post->ID : 0;
        $hash = md5($seed . 'global_limit');
        $hash_num = hexdec(substr($hash, 0, 8));
        $global_limit = $min + $hash_num % ($max - $min + 1);
    } else {
        $global_limit = 1;
    }

    if ($global_limit < 1) {
        $global_limit = PHP_INT_MAX;
    }

    $selected_candidates = array_slice($candidates, 0, $global_limit);

    // Group replacements by part index
    $replacements_by_part = array();
    foreach ($selected_candidates as $cand) {
        $pi = $cand['part_index'];
        if (!isset($replacements_by_part[$pi])) {
            $replacements_by_part[$pi] = array();
        }
        $replacements_by_part[$pi][] = $cand;
    }

    // Sort replacements within each part from last to first
    foreach ($replacements_by_part as $pi => &$list) {
        usort($list, function($a, $b) {
            return $b['local_offset'] - $a['local_offset'];
        });
    }
    unset($list);

    // Apply replacements
    foreach ($replacements_by_part as $pi => $cands) {
        $text = $parts[$pi];
        foreach ($cands as $cand) {
            $replacement = '<a href="' . esc_url($cand['url']) . '">' . $cand['match_text'] . '</a>';
            $text = substr_replace(
                $text,
                $replacement,
                $cand['local_offset'],
                strlen($cand['match_text'])
            );
        }
        $parts[$pi] = $text;
    }

    $processed_content = implode('', $parts);
    
    // Cache the processed content for 12 hours
    set_transient($cache_key, $processed_content, 12 * HOUR_IN_SECONDS);
    linkloom_track_transient($cache_key);
    
    return $processed_content;
}
