<?php

/**
 * Batch processing functions for LinkMap
 *
 * @package LinkLoom
 * @subpackage LinkMap
 */

if (!defined('ABSPATH')) {
    exit;
}

/**
 * Extract both single words and meaningful two-word phrases from content.
 *
 * @param string $content The content to process.
 * @return array Array of words and phrases.
 */
function linkloom_extract_words_and_phrases($content) {
    // Clean and normalize content
    $content = strtolower(trim($content));
    $content = preg_replace('/[^a-z0-9\s-]/', ' ', $content);
    
    // Split into words
    $words = preg_split('/\s+/', $content);
    $results = array();
    
    // Process words and phrases
    for ($i = 0; $i < count($words); $i++) {
        // Single word
        if (strlen($words[$i]) > 3) {
            $results[] = $words[$i];
        }
        
        // Two-word phrases
        if (isset($words[$i + 1])) {
            $phrase = $words[$i] . ' ' . $words[$i + 1];
            // Only include meaningful phrases (longer than 7 chars)
            if (strlen($phrase) > 7) {
                $results[] = $phrase;
            }
        }
    }
    
    return array_unique($results);
}

/**
 * Process a batch of posts for LinkMap generation.
 *
 * @param array $posts Array of WP_Post objects to process.
 * @param array $stop_words Array of common words to filter out.
 * @param array $existing_mappings Current keyword mappings.
 * @param int $batch_number Current batch number.
 * @param int $total_batches Total number of batches.
 * @return array Array containing frequency maps and page info.
 */
function linkloom_process_post_batch($posts, $stop_words, $existing_mappings, $batch_number, $total_batches) {
    $frequency_maps = array();
    $page_info = array();
    
    foreach ($posts as $post) {
        // Get raw content
        $content = wp_strip_all_tags(get_post_field('post_content', $post->ID));
        
        // Extract words and phrases
        $words = linkloom_extract_words_and_phrases($content);
        
        // Filter words and phrases
        $filtered_words = array_filter($words, function($word) use ($stop_words) {
            if ($word !== null && strpos($word, ' ') !== false) {
                // For phrases, check if either word is a stop word
                $parts = explode(' ', $word);
                return !in_array($parts[0], $stop_words) && 
                       !in_array($parts[1], $stop_words) && 
                       strlen($word) > 7;
            }
            // Original single word check
            return !in_array($word, $stop_words) && strlen($word) > 3;
        });
        
        // Count frequency
        $freq = array_count_values($filtered_words);
        
        // Store frequency map and page info
        $frequency_maps[$post->ID] = $freq;
        $page_info[$post->ID] = array(
            'page_id' => $post->ID,
            'page_title' => get_the_title($post->ID),
            'url' => get_permalink($post->ID)
        );
        
        // Update progress
        $current_post = array_search($post, $posts) + 1;
        $batch_progress = ($current_post / count($posts)) * 100;
        $overall_progress = (($batch_number - 1) * 100 + $batch_progress) / $total_batches;
        
        // Store progress in a transient
        $progress_key = 'linkloom_progress';
        set_transient($progress_key, array(
            'batch' => $batch_number,
            'total_batches' => $total_batches,
            'progress' => $overall_progress,
            'current_post' => $current_post,
            'total_posts' => count($posts)
        ), 30);
    }
    
    return array($frequency_maps, $page_info);
}

/**
 * Get common English stop words and phrases to filter out from keyword analysis.
 *
 * @return array Array of stop words and phrases.
 */
function linkloom_get_stop_words() {
    $basic_stop_words = array(
        'a', 'an', 'and', 'are', 'as', 'at', 'be', 'by', 'for', 'from', 'has', 'he',
        'in', 'is', 'it', 'its', 'of', 'on', 'that', 'the', 'to', 'was', 'were',
        'will', 'with', 'the', 'this', 'but', 'they', 'have', 'had', 'what', 'when',
        'where', 'who', 'which', 'why', 'how', 'all', 'any', 'both', 'each', 'few',
        'more', 'most', 'other', 'some', 'such', 'than', 'too', 'very', 'can', 'just',
        'should', 'now', 'i', 'you', 'your', 'we', 'my', 'me', 'her', 'his', 'their',
        'our', 'us', 'am', 'been', 'being', 'do', 'does', 'did', 'doing', 'would',
        'could', 'might', 'must', 'shall', 'into', 'if', 'then', 'else', 'out'
    );

    // Add common but meaningless phrases
    $phrase_stop_words = array(
        'more information',
        'read more',
        'click here',
        'learn more',
        'find out',
        'next page',
        'previous page',
        'first page',
        'last page',
        'contact us',
        'about us',
        'privacy policy',
        'terms conditions',
        'terms service',
        'all rights',
        'rights reserved'
    );

    return array_merge($basic_stop_words, $phrase_stop_words);
}
