<?php
/**
 * Main LinkMap generation functionality
 *
 * @package LinkLoom
 * @subpackage LinkMap
 */

if (!defined('ABSPATH')) {
    exit;
}

/**
 * Generate candidate mapping options for LinkMap by analyzing actual page content.
 *
 * @return array Array of candidate mappings.
 */
function linkloom_generate_linkmap() {
    // Get stop words
    $stop_words = linkloom_get_stop_words();

    // Get total number of posts
    $count_query = new WP_Query(array(
        'post_type' => array('post', 'page'),
        'post_status' => 'publish',
        'posts_per_page' => -1,
        'fields' => 'ids'
    ));
    $total_posts = $count_query->found_posts;
    
    if ($total_posts === 0) {
        return array();
    }
    
    // Process posts in batches of 10
    $batch_size = 10;
    $total_batches = ceil($total_posts / $batch_size);
    $frequency_maps = array();
    $page_info = array();
    
    // Get existing mappings from settings
    $settings = get_option('linkloom_settings', array());
    $existing_mappings = isset($settings['mappings']) ? $settings['mappings'] : array();
    
    for ($batch = 1; $batch <= $total_batches; $batch++) {
        // Get batch of posts
        $query = new WP_Query(array(
            'post_type' => array('post', 'page'),
            'post_status' => 'publish',
            'posts_per_page' => $batch_size,
            'paged' => $batch
        ));
        
        if (empty($query->posts)) {
            continue;
        }
        
        // Process batch
        list($batch_freq_maps, $batch_page_info) = linkloom_process_post_batch(
            $query->posts,
            $stop_words,
            $existing_mappings,
            $batch,
            $total_batches
        );
        
        // Merge results
        $frequency_maps = array_merge($frequency_maps, $batch_freq_maps);
        $page_info = array_merge($page_info, $batch_page_info);
    }
    
    // Clear progress transient
    delete_transient('linkloom_progress');

    // Build and analyze link graph
    $graph_data = linkloom_build_link_graph($page_info, $existing_mappings);
    $link_graph = $graph_data['graph'];
    $url_to_id = $graph_data['url_to_id'];
    $id_to_url = $graph_data['id_to_url'];
    
    // Find reachable nodes for each node
    $reachable_nodes = linkloom_analyze_reachability($link_graph);

    // Generate candidate mappings
    $candidates = array();
    foreach ($frequency_maps as $from_id => $from_freq) {
        $from_url = $page_info[$from_id]['url'];
        $from_node = $url_to_id[$from_url];
        
        // Skip if this page already has an outbound link
        if (!empty($link_graph[$from_node])) {
            continue;
        }

        foreach ($frequency_maps as $to_id => $to_freq) {
            if ($from_id === $to_id) {
                continue; // Skip self-links
            }

            $to_url = $page_info[$to_id]['url'];
            $to_node = $url_to_id[$to_url];
            
            // Skip if target already has inbound links or would create a cycle
            if (!empty(array_filter($link_graph, function($edges) use ($to_node) { 
                return in_array($to_node, $edges); 
            })) || linkloom_would_create_cycle($reachable_nodes, $from_node, $to_node)) {
                continue;
            }

            // Find common keywords between pages
            $common_words = array_intersect_key($from_freq, $to_freq);
            
            // Sort by frequency (highest in target page)
            arsort($common_words);
            
            // Take top 5 keywords
            $top_keywords = array_slice($common_words, 0, 5, true);
            
            foreach ($top_keywords as $keyword => $freq) {
                // Skip if this keyword mapping already exists
                $exists = false;
                foreach ($existing_mappings as $existing_keyword => $mapping) {
                    if (is_array($mapping) && 
                        $mapping['urlfrom'] === $page_info[$from_id]['url'] && 
                        $mapping['urlto'] === $page_info[$to_id]['url'] && 
                        $existing_keyword === $keyword) {
                        $exists = true;
                        break;
                    }
                }
                if (!$exists) {
                    $candidates[] = array(
                        'page_id' => $from_id,
                        'page_title' => $page_info[$from_id]['page_title'],
                        'keyword' => $keyword,
                        'urlto' => $page_info[$to_id]['url'],
                        'urlfrom' => $page_info[$from_id]['url']
                    );
                }
            }
        }
    }

    // Sort candidates by page title for better organization
    usort($candidates, function($a, $b) {
        return strcmp($a['page_title'], $b['page_title']);
    });

    return $candidates;
}
