<?php

/**
 * Graph analysis functions for LinkMap
 *
 * @package LinkLoom
 * @subpackage LinkMap
 */

if (!defined('ABSPATH')) {
    exit;
}

/**
 * Find all reachable nodes in a graph using Depth-First Search (DFS).
 *
 * @param array $graph Adjacency list representation of the link graph.
 * @param int $start Starting node ID.
 * @return array Array of reachable node IDs.
 */
function linkloom_find_reachable_nodes($graph, $start) {
    $visited = array();
    $stack = array($start);
    
    while (!empty($stack)) {
        $node = array_pop($stack);
        if (!isset($visited[$node])) {
            $visited[$node] = true;
            if (isset($graph[$node])) {
                foreach ($graph[$node] as $neighbor) {
                    if (!isset($visited[$neighbor])) {
                        $stack[] = $neighbor;
                    }
                }
            }
        }
    }
    
    return array_keys($visited);
}

/**
 * Build a link graph from existing mappings.
 *
 * @param array $page_info Array of page information.
 * @param array $existing_mappings Current keyword mappings.
 * @return array Array containing graph data structures.
 */
function linkloom_build_link_graph($page_info, $existing_mappings) {
    $link_graph = array();
    $url_to_id = array();
    $id_to_url = array();
    
    // First, build URL to ID mappings for easier graph processing
    foreach ($page_info as $pid => $info) {
        $url = $info['url'];
        $id = count($url_to_id);
        $url_to_id[$url] = $id;
        $id_to_url[$id] = $url;
        $link_graph[$id] = array();
    }
    
    // Build the graph edges from existing mappings
    foreach ($existing_mappings as $keyword => $mapping) {
        if (is_array($mapping)) {
            // For three-part mappings (keyword|URLto|URLfrom)
            $from_id = isset($url_to_id[$mapping['urlfrom']]) ? $url_to_id[$mapping['urlfrom']] : null;
            $to_id = isset($url_to_id[$mapping['urlto']]) ? $url_to_id[$mapping['urlto']] : null;
            if ($from_id !== null && $to_id !== null) {
                $link_graph[$from_id][] = $to_id;
            }
        } else {
            // For two-part mappings (keyword|URL)
            $to_url = trim($mapping);
            if (!empty($to_url)) {
                $to_id = isset($url_to_id[$to_url]) ? $url_to_id[$to_url] : null;
                if ($to_id !== null) {
                    // Find pages containing this keyword
                    foreach ($page_info as $pid => $info) {
                        $content = strtolower(wp_strip_all_tags(get_post_field('post_content', $pid)));
                        if ($content !== null && $keyword !== null && stripos($content, strtolower($keyword)) !== false) {
                            $from_id = $url_to_id[$info['url']];
                            $link_graph[$from_id][] = $to_id;
                        }
                    }
                }
            }
        }
    }
    
    return array(
        'graph' => $link_graph,
        'url_to_id' => $url_to_id,
        'id_to_url' => $id_to_url
    );
}

/**
 * Find all reachable nodes for each node in the graph.
 *
 * @param array $graph Link graph adjacency list.
 * @return array Array of reachable nodes for each node.
 */
function linkloom_analyze_reachability($graph) {
    $reachable_nodes = array();
    foreach (array_keys($graph) as $node) {
        $reachable_nodes[$node] = linkloom_find_reachable_nodes($graph, $node);
    }
    return $reachable_nodes;
}

/**
 * Check if adding an edge would create a cycle in the graph.
 *
 * @param array $reachable_nodes Current reachability map.
 * @param int $from_node Source node.
 * @param int $to_node Target node.
 * @return bool True if adding the edge would create a cycle.
 */
function linkloom_would_create_cycle($reachable_nodes, $from_node, $to_node) {
    // Check if target is already reachable from source (would create a cycle)
    if (in_array($to_node, $reachable_nodes[$from_node])) {
        return true;
    }
    
    // Check if source is already reachable from target (would create a cycle)
    if (in_array($from_node, $reachable_nodes[$to_node])) {
        return true;
    }
    
    return false;
}

/**
 * Get link graph data for visualization.
 *
 * @return array Formatted graph data for visualization.
 */
function linkloom_get_visualization_data() {
    // Get existing mappings
    $settings = get_option('linkloom_settings', array());
    $existing_mappings = isset($settings['mappings']) ? $settings['mappings'] : array();
    
    // Get page info for all published posts and pages
    $query = new WP_Query(array(
        'post_type' => array('post', 'page'),
        'post_status' => 'publish',
        'posts_per_page' => -1,
        'fields' => 'ids'
    ));
    
    $page_info = array();
    foreach ($query->posts as $post_id) {
        $page_info[$post_id] = array(
            'page_id' => $post_id,
            'page_title' => get_the_title($post_id),
            'url' => get_permalink($post_id)
        );
    }
    
    // Build the link graph
    $graph_data = linkloom_build_link_graph($page_info, $existing_mappings);
    
    // Format data for visualization
    $nodes = array();
    $edges = array();
    
    foreach ($page_info as $pid => $info) {
        $node_id = $graph_data['url_to_id'][$info['url']];
        $nodes[] = array(
            'id' => $node_id,
            'label' => $info['page_title'],
            'url' => $info['url'],
            'type' => get_post_type($pid)
        );
    }
    
    foreach ($graph_data['graph'] as $from_id => $to_ids) {
        foreach ($to_ids as $to_id) {
            $edges[] = array(
                'from' => $from_id,
                'to' => $to_id
            );
        }
    }
    
    return array(
        'nodes' => $nodes,
        'edges' => $edges
    );
}

// Register AJAX handler
function linkloom_register_visualization_ajax() {
    add_action('wp_ajax_linkloom_get_visualization_data', function() {
        check_ajax_referer('linkloom-linkmap-nonce', 'nonce');
        
        if (!current_user_can('manage_options')) {
            wp_send_json_error('Unauthorized');
        }
        
        $data = linkloom_get_visualization_data();
        wp_send_json_success($data);
    });
}
add_action('init', 'linkloom_register_visualization_ajax');
